#include "flashLED.h"

uint8_t flashLED::pin;
uint8_t flashLED::flashes;
uint16_t flashLED::onTime;
uint16_t flashLED::offTime;
uint16_t flashLED::finalOffTime;
uint8_t flashLED::flashesRemaining;
uint16_t flashLED::onRemaining;
uint16_t flashLED::offRemaining;
uint32_t flashLED::prev; // millis() value from previous loop
uint8_t flashLED::mode; // flashing, off, or on

void flashLED::start(uint8_t outputPin, uint16_t onMs, uint16_t offMs, uint16_t finalOffMs) {
  pin = outputPin;
  pinMode(pin, OUTPUT);
  digitalWrite(pin, LOW);
  flashes = mode = 0;
  onTime = onMs;
  offTime = offMs;
  offRemaining = finalOffTime = finalOffMs;
  onRemaining = 0;
  flashesRemaining = 0;
  prev = millis();
}

void flashLED::set(uint8_t n) {
  flashes = n;
}

void flashLED::off(void) {
  digitalWrite(pin, LOW); // LED off
  mode = 1;
}

void flashLED::on(void) {
  digitalWrite(pin, HIGH); // LED on
  mode = 2;
}


void flashLED::loop(void) {
  uint32_t now = millis();
  uint16_t elapsed = (uint16_t)(now - prev);
  prev = now;

  if (!mode) {
    if (onRemaining) { // LED currently on
      if (elapsed >= onRemaining) { // time to switch
        digitalWrite(pin, LOW); // LED off
        onRemaining = 0;
        elapsed -= onRemaining; // elapsed now has remaining of 'off' time
        if (flashesRemaining == 0) {
          offRemaining = finalOffTime;
          if (flashes) {
            flashesRemaining = flashes - 1;
          } else {
            flashes = 0;
          }
        } else {
          offRemaining = offTime;
          flashesRemaining--;
        }
        
        if (offRemaining > elapsed) {
          offRemaining -= elapsed;
        } else {
          offRemaining = 1;
        }
      } else {
        onRemaining -= elapsed;
      }
    } else { // LED currently off
      if (elapsed >= offRemaining) { // time to switch
        if (flashes) {
          digitalWrite(pin, HIGH); // LED on
          offRemaining = 0;
          elapsed -= offRemaining; // elapsed now has remaining of 'off' time
          if (onTime > elapsed) {
            onRemaining = onTime - elapsed;
          } else {
            onRemaining = 1;
          }
        } else {
          offRemaining = finalOffTime;
        }
      } else {
        offRemaining -= elapsed;
      }
    }
  }  
}

